/***************************************************************

  BayCom(R)        Packet-Radio fuer IBM PC

  OpenBayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------
  Diverse Utilities
  -----------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980118 OE3DZW only 2 digits for year in date (02 instead of 102 for 2002)
//                seconds in timestr() fn, new parameter sec
//19980505 OE3DZW added rmcrlf
//19990110 OE3DZW y2k: parse_time,zeitstring
//19990207 OE3DZW added new datestr-type
//19990615 DH3MB  Moved getmd2sum, getmd5sum, mdsum2str to mbutil.cpp
//199911.. DF3VI  nextword(): wie nexttoken(), aber ohne "", an vielen stellen sicherer
//20000408 DF3VI  parseline(): aus mbfwd.cpp
//20000427 DK2UI  new datestr(), removed zeitstring() and timestr()
//20000524 DK2UI  parse_time() moves pointer behind end of string, corrected
//20000518 DK2UI  new setfiletime() and gethextime()
//20000615 DK2UI  filetime() returns 0L if not a regular file (e.g. a dir)

#include "baycom.h"

//#define UTC_TIME      //for testing UTC output

/*---------------------------------------------------------------------------*/

unsigned strcrc (char *s)
//*************************************************************************
//
//  Berechnet ueber den gegebenen String eine CRC-Pruefsumme
//  Wird fr Hashing verwendet
//
//*************************************************************************
{ short int word = 0;
  unsigned int i;
  byte byt;

  while ((byt = *(s++)) != 0)
  { for (i = 0; i < 8; i++)
    { if (((word < 0) + (byt < 128)) == 1) word <<= 1;
      else word = (word << 1) ^ 0xa097;
      byt <<= 1;
    }
  }
  return (short unsigned) word;
}

/*---------------------------------------------------------------------------*/

char *stristr (char *s1, char *s2)
//*************************************************************************
//
//  Sucht nach einem Teilstring in einem String.
//  Die Funktion ist analog zur Bibliotheksfunktion strstr,
//  unterscheidet aber nicht zwischen Gross - und Kleinschreibung
//
//*************************************************************************
{ int i;
  int l = strlen (s2);

  while (s1[0])
  { i = 0;
    while (( ! ((s1[i] ^ s2[i]) & 0x5f)) && s2[i]) i++;
    if (i == l) return s1;
    s1++;
  }
  return NULL;
}

/*---------------------------------------------------------------------------*/

//*************************************************************************
//
//  strpos(String,Substring)
//
//  Gibt die Position von Substring in String zurueck
//  Wird Substring nicht gefunden, oder ist einer der beiden Strings leer,
//  so wird -1 zurckgegeben.
//
//*************************************************************************
int strpos (char *s1, char *s2)
{ unsigned int i = 0;
  unsigned int l = strlen (s2);
  int pos = 0;

  while (*s1)
  {
    while (( ! ((s1[i] ^ s2[i]) & 0x5f)) && s2[i]) i++;
    if (i == l) return pos;
    s1++;
    pos++;
  }
  return NOTFOUND;
}

/*---------------------------------------------------------------------------*/

char *nexttoken (char *buf, char *token, int maxlen)
//*************************************************************************
//
//  Extrahiert den naechsten alphanumerischen Teilstring aus einem
//  String. Dabei wird entweder blank/tab/@ als Ende betrachtet, oder
//  der String kann mit "Anfuehrungszeichen" eingeklammert sein
//
//*************************************************************************
{ int i = 0;

  while (*buf == ' ') buf++;
  if (*buf == '"')
  { buf++;
    while ((*buf != '"') && (*buf >= ' ') && (i < maxlen)) token[i++] = *(buf++);
    while (*buf && (*buf) != '"') buf++;
    if (*buf == '"') buf++;
  }
  else
  { while (*buf != '@' && *buf > ' ' && i < maxlen) token[i++] = *(buf++);
    while (*buf && *buf != '@' && *buf != ' ') buf++;
  }
  while (*buf == ' ') buf++;
  token[i] = 0;
  return buf;
}

/*---------------------------------------------------------------------------*/

char *nextword (char *buf, char *token, int maxlen)
//*************************************************************************
//
//  Extrahiert den naechsten alphanumerischen Teilstring aus einem
//  String. Dabei wird entweder blank oder NULL als Ende betrachtet.
//
//*************************************************************************
{ int i = 0;
  char *buf2 = buf; //do not modify argument

  while (*buf2 == ' ') buf2++;
  while (*buf2 > ' ' && i < maxlen) token[i++] = *(buf2++);
  token[i] = 0;
  while (*buf2 == ' ') buf2++;
  return buf2;
}

/*---------------------------------------------------------------------------*/

int parseline (char *s, char *indizes) //df3vi: aus mbfwd.cpp
//*************************************************************************
//
//  Teilt eine Zeile in einzelne Woerter auf.
//  Rueckgabe: Anzahl der Woerter
//             Array mit den Indizes der einzelnen Woerter
//
//*************************************************************************
{ int offset = 0, i = 0, word = 0;
  while (*s >= ' ')
  { if (*s == ' ' || *s == ',')
    { *s = 0;
      word = 0;
    }
    else
    { if ( ! word)
      {
        if ((i + 1) < MAXPARSE) indizes[i++] = offset;
        else break;
      }
      word = 1;
    }
    s++;
    offset++;
  }
  *s = 0;
  indizes[i] = 0;
  return i;
}

/*---------------------------------------------------------------------------*/

char *get_searchitem (char *instr, char *str, int maxlen)
//*************************************************************************
//
//  Extrahiert in einer Befehlszeile einen Suchbegriff in
//  "Anfuehrungszeichen". Dieser wird herauskopiert und gegen Blanks
//  ersetzt. Blanks am Zeilenende werden entfernt.
//
//*************************************************************************
{ char *qm = strchr (instr, '"');

  if (qm)
  { char *end = strchr (qm + 1, '"');
    if (end)
    { while (qm != end)
      { *qm = ' ';
        qm++;
        if (maxlen && qm != end)
        { *str = toupper (*qm);
          str++;
          maxlen--;
        }
      }
      *str = 0;
      *end = ' ';
      while (*instr && instr [strlen (instr) - 1] == ' ')
        instr [strlen (instr) - 1] = 0;
      return str;
    }
  }
  return NULL;
}

/*---------------------------------------------------------------------------*/

char *datestr (time_t utime, int ymode) //dk2ui
//*************************************************************************
//
//  Legt das Datum in einem String ab
//
//  0 .. without year, used for check, list     03.11
//  1 .. 2 digit year, (for compatiblity)       03.11.00
//  2 .. 4 digit-year, (where possible)         03.11.2000
//  3 .. hour and minute                        15:09z
//  4 .. hour, minute and seconds               15:09:17z
//  5 .. 2 digit-year, used for R:-header       001103/1509
//  6 .. 4 digit-year, used for binsplit        20001103150917
//  7 .. 2 digit-year, used for forward         0311001509
//  8 .. 2 digit-year, used for WP-mails        001103
//  9 .. 4 digit-year, used for dirformat       20001103
//  10.. 2 digit year, without seconds          03.11.00 15:09z
//  11.. 2 digit year, with seconds             03.11.00 15:09:17z
//  12.. 4 digit year, with seconds             03.11.2000 15:09:17z
//  13.. RFC-date (obsolete) use ISO            Mon, 07 Jul 1997 20:13:48
//  14.. 4 digit-year, ISO 8601/EN28601         Fri, 2000-Nov-03 15:09:07z
//  15.. 4 digit-year, ISO 8601/EN28601         2000-11-03 15:09:07z
//  16.. 1 digit-year, used for log file        01103
//  17.. 2 digit-month+day, 2digithour+minute   1023/1635
//
//*************************************************************************
{ static char datum[27];
#ifdef UTC_TIME
  int local = FALSE;
#else
  int local = ad_timezone () ? TRUE : FALSE;
#endif
#ifdef OLDTIMEFMT
  #define TZ 32
#else
  #define TZ local?'l':'z'
#endif
  static char *wd[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
  static char *mn[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                        "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
  struct tm *tt = ad_comtime (local ? utime + ad_timezone () : utime);
  if (tt)
  {
    switch (ymode) {
      case 0: sprintf (datum, "%02d.%02d",
                tt->tm_mday, tt->tm_mon + 1); break;
      case 1: sprintf (datum, "%02d.%02d.%02d",
                tt->tm_mday, tt->tm_mon + 1, tt->tm_year % 100); break;
      case 2: sprintf (datum, "%02d.%02d.%04d",
                tt->tm_mday, tt->tm_mon + 1, tt->tm_year + Year); break;
      case 3: sprintf (datum, "%02d:%02d%c",
                tt->tm_hour, tt->tm_min, TZ); break;
      case 4: sprintf (datum, "%02d:%02d:%02d%c",
                tt->tm_hour, tt->tm_min, tt->tm_sec, TZ); break;
      case 5: sprintf (datum, "%02d%02d%02d/%02d%02d",
                tt->tm_year % 100, tt->tm_mon + 1, tt->tm_mday,
                tt->tm_hour, tt->tm_min); break;
      case 6: sprintf (datum, "%04d%02d%02d%02d%02d%02d",
                tt->tm_year + Year, tt->tm_mon + 1, tt->tm_mday,
                tt->tm_hour, tt->tm_min, tt->tm_sec); break;
      case 7: sprintf (datum, "%02d%02d%02d%02d%02d",
                tt->tm_mday, tt->tm_mon + 1, tt->tm_year % 100,
                tt->tm_hour, tt->tm_min); break;
      case 8: sprintf (datum, "%02d%02d%02d",
                tt->tm_year % 100, tt->tm_mon + 1, tt->tm_mday); break;
      case 9: sprintf (datum, "%4d%02d%02d",
                tt->tm_year + Year, tt->tm_mon + 1, tt->tm_mday); break;
      case 10: sprintf (datum, "%02d.%02d.%02d %02d:%02d%c",
                tt->tm_mday, tt->tm_mon + 1, tt->tm_year % 100,
                tt->tm_hour, tt->tm_min, TZ); break;
      case 11: sprintf (datum, "%02d.%02d.%02d %02d:%02d:%02d%c",
                tt->tm_mday, tt->tm_mon + 1, tt->tm_year % 100,
                tt->tm_hour, tt->tm_min, tt->tm_sec, TZ); break;
      case 12: sprintf (datum, "%02d.%02d.%04d %02d:%02d:%02d%c",
                tt->tm_mday, tt->tm_mon + 1, tt->tm_year + Year,
                tt->tm_hour, tt->tm_min, tt->tm_sec, TZ); break;
      case 13: sprintf (datum, "%s, %d %s %d %02d:%02d:%02d",
                wd [tt->tm_wday], tt->tm_mday, mn [tt->tm_mon], tt->tm_year + Year,
                tt->tm_hour, tt->tm_min, tt->tm_sec); break;
      case 14: sprintf (datum, "%s, %04d-%s-%02d %02d:%02d:%02d%c",
                wd [tt->tm_wday], tt->tm_year + Year, mn [tt->tm_mon], tt->tm_mday,
                tt->tm_hour, tt->tm_min, tt->tm_sec, TZ); break;
      case 15: sprintf (datum, "%04d-%02d-%02d %02d:%02d:%02d%c",
                tt->tm_year + Year, tt->tm_mon + 1, tt->tm_mday,
                tt->tm_hour, tt->tm_min, tt->tm_sec, TZ); break;
      case 16: sprintf (datum, "%1d%02d%02d",
                tt->tm_year % 10, tt->tm_mon + 1, tt->tm_mday); break;
      case 17: sprintf (datum, "%02d%02d/%02d%02d",
                tt->tm_mon + 1, tt->tm_mday, tt->tm_hour, tt->tm_min); break;
      default: *datum = 0;
    }
  }
  else *datum = 0;
#undef TZ
  return datum;
}

/*---------------------------------------------------------------------------*/

int newdate (time_t d1, time_t d2)
//*************************************************************************
//
//  Vergleicht das Datum 2er Tage im UNIX-Format
//
//*************************************************************************
{ char datum[12];
  if (d1 && d2)
  { strcpy (datum, datestr (d1, 1));
    return strcmp (datum, datestr (d2, 1));
  }
  else return 1;
}

/*---------------------------------------------------------------------------*/

char *zeitspanne (time_t zeit, delta_t mode)
//*************************************************************************
//
//  Gibt einen Zeitunterschied als String zurueck.
//
//  Werte fuer mode:
//  zs_runtime  : Zeit (in sec) wird als [dd.]hh.mm  ausgegeben
//  zs_seconds  : Zeit (in sec) wird als [t]s [t]m [t]h [t]d ausgegeben
//  zs_cputime  : Zeit (in 55msec) wird wie <seconds> ausgegeben
//  zs_cpuexact : Zeit (in 55msec) wird dezimal sss.ms ausgegeben
//
//*************************************************************************
{ unsigned long hours = zeit / HOUR;
  unsigned long days = zeit / DAY;
  static char s[16];

  *s = 0;
  if (mode > zs_runtime)
  { if (mode >= zs_cputime)
    { zeit *= MS_PER_TIC;       // from now on take milliseconds
      if (mode == zs_cpuexact)
      { sprintf(s, "%u.%02u", (unsigned)(zeit / 1000), ((unsigned)(zeit % 1000) / 10));
        return s;
      }
      if (zeit < 10000)
      { sprintf(s, "%u.%us", (unsigned)(zeit / 1000), ((unsigned)(zeit % 1000)) / 100);
        return s;
      }
      zeit /= 1000L;                // too much for decimal display
    }
    if (zeit < 240L) sprintf (s, "%lds", zeit); // below 4min show seconds
    else if (zeit < (HOUR * 4))     // below 4h show minutes
      sprintf(s, "%ldm", zeit / 60);
    else if (zeit < (DAY * 4))
      sprintf(s, "%ldh", zeit / HOUR); // below 4days in show hours
    else if (zeit < (DAY * 365))
      sprintf(s, "%ldd", zeit / DAY); //from year on show in years
    else if (zeit < (DAY * 3650))
      sprintf(s, "%ldy", zeit / (DAY * 365));
  }
  else
  { if (days > 999UL) sprintf (s, "-:--");
    else
    { zeit -= hours * HOUR;
      hours -= days * 24UL;
      if (days) sprintf (s, "%lu.", days);
      sprintf (s + strlen (s), "%02u:%02u", (unsigned) hours, (unsigned) zeit / 60);
    }
  }
  return s;
}

/*---------------------------------------------------------------------------*/

time_t parse_time (char *datum)
//*************************************************************************
//
//  Wandelt einen String, der ein Lokal-Datum enthaelt, in eine
//  ANSI-UTC-Zeit (UTC) um.
//  Dabei werden unbekannte Teile mit dem aktuellen Datum ersetzt.
//
//*************************************************************************
{ struct tm tt;
  char *olddatum = datum;

  memcpy (&tt, ad_comtime (ad_time()), sizeof (struct tm));
  tt.tm_hour = 0;
  tt.tm_min = 0;
  tt.tm_sec = 0;
  tt.tm_isdst = -1;
  while (*datum && ! isalnum (*datum)) datum++;
  if (isdigit (*datum))
  { tt.tm_mday = atoi (datum);
    while (isdigit (*datum++));
    if (isdigit (*datum))
    { tt.tm_mon = atoi (datum) - 1;
      while (isdigit (*datum++));
      if (isdigit (*datum))
      { tt.tm_year = atoi (datum);
        if (tt.tm_year < 70) tt.tm_year += 100;
        if (tt.tm_year > Year) tt.tm_year -= Year; //y2k, accept 4 digit date
        while (isdigit (*datum)) datum++;
      }
    }
    if (tt.tm_mday < 1 || tt.tm_mday > 31 || tt.tm_mon < 0 || tt.tm_mon > 11)
    { putf (ms (m_date_invalid));
      return 0L;
    }
  }
  while ((olddatum != datum) && *olddatum)
  { *olddatum = ' ';
    olddatum++;
  }
  return ad_mktime (&tt) - ad_timezone ();
}

/*---------------------------------------------------------------------------*/

int filecopy (char *src, char *dest)
//*************************************************************************
//
//  Kopiert ein File
//  Returnwert  0 ok, -1 Fehler
//
//*************************************************************************
{ handle inf, outf;
  char *buf;
  unsigned z;

  inf = s_open (src, "srb");
  if (inf == EOF) { s_close (inf); return (-1); }
  outf = s_open (dest, "swb");
  if (outf == EOF) { s_close (outf); return (-1); }
  buf = (char *) t_malloc (CBUFLEN, "copy");
  do
  { z = _read (inf, buf, CBUFLEN);
    if (z > 0) _write (outf, buf, z);
    waitfor (e_ticsfull);
  } while (z == CBUFLEN);
  t_free (buf);
  s_close (outf);
  s_close (inf);
  return 0;
}

/*---------------------------------------------------------------------------*/

int xunlink (char *name)
//*************************************************************************
//
//  Loescht ein File unter Beruecksichtigung der Mehrfachzugriffe
//  Ggf. wird gewartet bis das File frei ist.
//
//*************************************************************************
{ while (sema_access (name)) wdelay (107);
  resetreadonly (name);
  return unlink (name);
}

/*---------------------------------------------------------------------------*/

int xrename (char *src, char *dest)
//*************************************************************************
//
//  Verschiebt ein File unter Beruecksichtigung der Mehrfachzugriffe
//  Ggf. wird gewartet bis das File frei ist.
//  Wenn Verschieben nicht moeglich ist wird kopiert und danach geloescht.
//
//*************************************************************************
{ while (sema_access (src)) wdelay (124);
  while (sema_access (dest)) wdelay (125);
  resetreadonly (src);
  if (rename (src, dest))
  { filecopy (src, dest);
    return unlink (src);
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

long filesize (char *name)
//*************************************************************************
//
//  Gibt die Groesse eines Files zurueck
//
//*************************************************************************
{ struct stat st;
  if ( ! stat (name, &st)) return st.st_size;
  else return 0L;
}

/*---------------------------------------------------------------------------*/

int isdir (char *name)
//*************************************************************************
//
//  Stellt fest, ob der uebergebene Name ein Verzeichnis ist
//
//*************************************************************************
{ struct stat st;
  if (! strcmp (name + strlen (name) - 2 , "..")) return 1; // workaround WIN32 Bug
  if (! stat (name, &st)) return ((st.st_mode & S_IFDIR) != 0);
  else return 0;
}

/*---------------------------------------------------------------------------*/

time_t filetime (char *name)
//*************************************************************************
//
//  Stellt die Zeit der letzten Modifikation eines Files oder Verzeichnisses fest
//  (wird auch zum Feststellen der Existenz eines Files oder Verzeichnisses verwendet)
//
//*************************************************************************
{
  struct stat st;
  if ( ! stat (name, &st)) return st.st_mtime;
  else return 0L;
}

/*---------------------------------------------------------------------------*/

time_t file_isreg (char *name)
//*************************************************************************
//
//  Stellt die Zeit der letzten Modifikation eines Files fest
//  (wird auch zum Feststellen der Existenz eines Files verwendet)
//
//*************************************************************************
{
  struct stat st;
  if ( ! stat (name, &st)
#ifdef __LINUX__
      && S_ISREG (st.st_mode))
#else
      && (st.st_mode & S_IFREG))
#endif
    return st.st_mtime;
  else return 0L;
}

/*---------------------------------------------------------------------------*/

char *defext (char *name, char *ext)
//*************************************************************************
//
//  Fuegt eine default extension zu einem Filename, sofern eine solche
//  noch nicht existiert.
//
//*************************************************************************
{ static char nam2 [FNAMELEN-1];
  strlwr (name);
  strlwr (ext);
  if (strchr (name, '.')) return name;
  strcpy (nam2, name);
  strcat (nam2, ".");
  strcat (nam2, ext);
  return nam2;
}

/*---------------------------------------------------------------------------*/

void killbackslash (char *s)
//*************************************************************************
//
//  Macht aus allen Backslashes einen Slash /
//  (Wird verwendet zur Umwandlung von DOS-Filepfaden nach UNIX)
//
//*************************************************************************
{ while (*s)
  { if (*s == '\\') *s = '/';
    s++;
  }
}

/*---------------------------------------------------------------------------*/

void remove_emptyadd (char *s)
//*************************************************************************
// DH8YMB
// Entfernt bei HTTPD aus dem TO-Feld ein leeres " @ "
// Bsp: aus "TEST @ " wird "TEST "
//*************************************************************************
{
  unsigned int i = 0;
  unsigned int leng = strlen (s);
  while (*s)
  { if (*s == '@' && (i == leng-2)) *s = 0;
    s++; i++;
  }
}

/*---------------------------------------------------------------------------*/

char jokcmp (char *string, char *jokex, char negotiation)
//*************************************************************************
//
//  Eine einfache Match-Routine (NICHT regex!), die die Verwendung der
//  Jokerzeichen '?' und '*' (auch mitten im String) erlaubt.
//  Stimmt "negotiation" mit dem ersten Zeichen von "jokex" ueberein,
//  so wird der Ausdruck negiert.
//  (DH3MB)
//
//*************************************************************************
{ char i1 = 0, i2 = 0, neg = 0;

  if (negotiation == *jokex)
  { neg = 1;
    jokex++;
  }
  for (;i2 < strlen (jokex); i2++)
  {
    switch (jokex[i2])
    {
    case '?':
      if (i1 == strlen (string)) return neg; // '?' stated, but no char left in string
      i1++;
      break;
    case '*':
      if (i2 == (strlen (jokex) - 1)) return 1 - neg; // '*' at right bound of jokex
      while (jokex[i2] == '*') i2++;
      while (string[i1] != jokex[i2] && jokex[i2] != '?')
      { i1++;
        if (i1 == strlen (string)) return neg;
      }
      i1++;
      break;
    default:
      if (string[i1] != jokex[i2]) return 0+neg;
      i1++;
      break;
    }
  }
  if (i1 == strlen (string)) return 1 - neg;
  else return neg;
}

/*---------------------------------------------------------------------------*/

/* not used yet
time_t parseISOtime (char *datums)      //dk2ui
//-------------------------------------------------------------------------
//
//  Wandelt einen String, der ein ISO-Datum(localtime) enthaelt in eine ANSI-Zeit(UTC) um.
//
//-------------------------------------------------------------------------
{ struct tm tt;
  char *datptr = datums;

  memset (&tt, 0, sizeof (struct tm));
  tt.tm_isdst = -1;
  while (*datptr && ! isalnum (*datptr)) datptr++;
  if (isdigit (*datptr))
  { tt.tm_year = atoi (datptr);
    if (tt.tm_year > Year) tt.tm_year -= Year;
    if (tt.tm_year < 70)   tt.tm_year += 100;
    while (isdigit (*datptr++));
    if (isdigit (*datptr))
    { tt.tm_mon = atoi (datptr) - 1;
      while (isdigit (*datptr++));
      if (isdigit (*datptr))
      { tt.tm_mday = atoi (datptr);
        while (isdigit (*datptr++));
        if (isdigit (*datptr))
        { tt.tm_hour = atoi (datptr);
          while (isdigit (*datptr++));
          if (isdigit (*datptr))
          { tt.tm_min = atoi (datptr);
            while (isdigit (*datptr++));
            if (isdigit (*datptr))
              tt.tm_sec = atoi (datptr);
          }
        }
      }
    }
    if (tt.tm_mday < 1 || tt.tm_mday > 31 || tt.tm_mon < 0 || tt.tm_mon > 11)
    { putf (ms (m_date_invalid));
      return 0L;
    }
  }
  return ad_mktime (&tt) - ad_timezone ();
}
*/
/*---------------------------------------------------------------------------*/

//dk2ui: the BIN file time format is from a Borland C struct and unknown in Linux
#ifdef __FLAT__
    struct ftime                // 32 bit
    {
      unsigned ft_tsec  : 5;    // 0..29 * 2 (!)
      unsigned ft_min   : 6;    // 0..59
      unsigned ft_hour  : 5;    // 0..23
      unsigned ft_day   : 5;    // 1..31
      unsigned ft_month : 4;    // 1..12
      unsigned ft_year  : 7;    // years from 1980
    };

#ifdef __LINUX__
 #include <utime.h>
#else   // _Win32
 #include <winbase.h> // FILETIME, DosDateTimeToFileTime, SetFileTime
#endif
#endif

#ifdef __FLAT__
time_t getunixtime (unsigned long dosfiletime)  // dk2ui
{
    struct ftime *ft;
    struct tm tt;
    ft = (struct ftime *) &dosfiletime;
    memset (&tt, 0, sizeof (struct tm));
    tt.tm_isdst = -1;
    tt.tm_sec  = ft->ft_tsec << 1;
    tt.tm_min  = ft->ft_min;
    tt.tm_hour = ft->ft_hour;
    tt.tm_mday = ft->ft_day;
    tt.tm_mon  = ft->ft_month - 1;
    tt.tm_year = ft->ft_year + 80;
    return ad_mktime (&tt); // localtime
}
#endif

int setfiletime (char *filename, unsigned long dosfiletime)     //dk2ui
{
  if ( ! dosfiletime) return FALSE;
#ifdef __FLAT__
#ifdef __LINUX__
    struct utimbuf ut;
    ut.actime = ut.modtime = getunixtime (dosfiletime);
    if ( ! utime (filename, &ut)) return TRUE;
    return FALSE;
#else   // _WIN32
    HANDLE fh = OpenFileMapping (FILE_MAP_WRITE, TRUE, filename);
    if ( ! fh) return FALSE;
    FILETIME wfiletime;
    LPFILETIME wft = &wfiletime;
    if ( ! DosDateTimeToFileTime (
      (unsigned int)((dosfiletime & 0xffff0000) >> 16),
      (unsigned int) (dosfiletime & 0x0000ffff), wft))
      return FALSE;
    int result = SetFileTime (fh, wft, wft, wft);
    CloseHandle (fh);
    return result;
#endif
#else   // DOS16
    int fh;
    union fzeit { ftime ft; unsigned long fz;} fzeit;
    fzeit.fz = dosfiletime;
    fh = s_open (filename, "srb");
    setftime (fh, &fzeit.ft);
    s_close (fh);
    return TRUE;
#endif
}

// returns a 32 bit value in Borland C ftime format for #BIN#
unsigned long getdostime (time_t timeu) //dk2ui
{
  static union u2d { struct ftime ft; unsigned long dostime; } ud;

  if ( ! timeu) return 0L;
    struct tm *tt = ad_comtime (timeu);
    ud.ft.ft_tsec  = tt->tm_sec >> 1;
    ud.ft.ft_min   = tt->tm_min;
    ud.ft.ft_hour  = tt->tm_hour;
    ud.ft.ft_day   = tt->tm_mday;
    ud.ft.ft_month = tt->tm_mon + 1;
    ud.ft.ft_year  = tt->tm_year - 80;
  return ud.dostime;
}

/*---------------------------------------------------------------------------*/
